/*
 * Picviz - Parallel coordinates ploter
 * Copyright (C) 2008 Sebastien Tricaud <toady@gscore.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $Id: pcv.c 289 2008-10-27 12:33:58Z toady $
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <picviz.h>

/*
 * Copy arg vector into a new buffer, concatenating arguments with spaces.
 * stolen from tcpdump
 */
char *
concat_argv(char **argv)
{
	char **p;
	u_int len = 0;
	char *buf;
	char *src, *dst;

	p = argv;
	if (*p == 0)
		return 0;

	while (*p)
		len += strlen(*p++) + 1;

	buf = (char *)malloc(len);
	if (buf == NULL) {
		fprintf(stderr, "concat_argv: malloc");
		return NULL;
	}

	p = argv;
	dst = buf;
	while ((src = *p++) != NULL) {
		while ((*dst++ = *src++) != '\0')
			;
		dst[-1] = ' ';
	}
	dst[-1] = '\0';

	return buf;
}

void help(char *prg)
{
	fprintf(stderr, "Usage: %s -Tplugin [-Rplugin] [-Warg] [-ad] [-A argument] file.pcv ['filter']\nread the manpage pcv(1) for details\n",
			prg);
	exit(EXIT_FAILURE);
}

int main(int argc, char **argv)
{
	struct pcimage_t *image;
	char *filename;
	char *filter = NULL;
	char plugin_out_str[1024];
	char plugin_ren_str[1024];
	char *plugin_arg = NULL;
	char plugin_in = 0;
	char plugin_ren_in = 0;
	char draw_heatline = 0;
	int opt;
	char debug = 0;
	unsigned int height = 0;
	unsigned int axis_default_space = 0;

	picviz_init();

	while ((opt = getopt(argc, argv, "lrT:R:W:daA:")) != -1) {
		switch (opt) {
			case 'r':
				height += 200;
				axis_default_space += 100;
				break;
			case 'd':
				debug = 1;
				break;
			case 'A':
				plugin_arg = optarg;
				break;
			case 'T':
				sprintf(plugin_out_str, "libpicvizout%s.so", optarg);
				plugin_in = 1;
				break;
			case 'R':
				sprintf(plugin_ren_str, "libpicvizren%s.so", optarg);
				if ( ! strcmp("healine", optarg) ) {
					draw_heatline = 1;
				}
				plugin_ren_in = 1;
				break;
			case 'l':
				engine.learn = 0;
				break;
			case 'a':
				engine.display_raw_data = 1;
				break;
			case 'W':
				if ( ! strcmp("pcre", optarg)) {
					engine.use_pcre = 1;
					break;
				}
				if ( ! strcmp("string_algo_basic", optarg)) {
					engine.string_algo = 0;
					break;
				}
				if ( ! strcmp("debug", optarg)) {
					engine.debug = 1;
					break;
				}
				fprintf(stderr,"ERROR: unrecognized -W%s.\n", optarg);
				fprintf(stderr,"ERROR: unknown engine parameter!\n");
				help(argv[0]);
				break;
			default: /* '?' */
				fprintf(stderr,"ERROR: arguments parsing!\n");
				help(argv[0]);
		}
	}

	if (optind >= argc) {
		fprintf(stderr, "File to parse missing\n");
		exit(EXIT_FAILURE);
	}

	if (!plugin_in) {
		fprintf(stderr,"ERROR: plugin required!\n");
		help(argv[0]);
	}

	filename = argv[optind];
	filter = concat_argv(&argv[optind+1]);

	engine.axis_default_space += axis_default_space;
	engine.image_height += height;
	image = (struct pcimage_t *)pcv_parse(filename, filter);
	if (!image) {
		fprintf(stderr, "Cannot parse image %s\n", argv[optind]);
		exit(EXIT_FAILURE);
	}

	if (debug) {
		picviz_image_debug_printall(image);
		goto out;
	}

	if (plugin_ren_in) {
		picviz_plugin_load(PLUGIN_TYPE_RENDER, plugin_ren_str, image, plugin_arg);
	}
	picviz_plugin_load(PLUGIN_TYPE_OUTPUT, plugin_out_str, image, plugin_arg);

out:
	picviz_image_destroy(image);

	exit(EXIT_SUCCESS);
}
