/*
 * props/color.c: Manage [color="red"] property
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <picviz.h>
#include "color.h"
#include "utils.h"

char *picviz_color_named_to_hexstr(char *color)
{
        int MAX_COLORS = 13;
        char *pcolors[] = {"white","black", "red", "green", "blue", "yellow", "grey", "turquoise", "pink", "orange", "darkblue", "darkgreen", "darkred", "brown"};
        char *chart[] = {"#FFFFFF","#000000", "#FF0000", "#00FF00", "#0000FF", "#FFFF00", "#C0C0C0", "#00FFFF", "#FF0099", "#FF9900", "#3333CC", "#339933", "#990000", "#8B6969"};
        int i=0;

        if (color[0] == '#')
            return picviz_string_up(color);

        for (i=0;i<=MAX_COLORS;i++) {
#ifdef DEBUG_PROPERTIES
                printf("color=[%s];pcolors[i]=[%s], value=[%s]\n", color, pcolors[i], chart[i]);
#endif
                if (!strcmp(color, pcolors[i])) {
                        return strdup(chart[i]);
                }
        }

        return strdup("#000000"); /* Color not found, we return black as default */
}


float picviz_color_extract_r(char *s)
{
        int i;
        char p[3];

        p[0] = s[1];
        p[1] = s[2];
        p[2] = '\0';

        i = strtol(p, NULL, 16);

        return (float) i / 255;
}

float picviz_color_extract_g(char *s)
{
        int i;
        char p[3];

        p[0] = s[3];
        p[1] = s[4];
        p[2] = '\0';

        i = strtol(p, NULL, 16);

        return (float) i / 255;
}

float picviz_color_extract_b(char *s)
{
        char p[3];
        int i;

        p[0] = s[5];
        p[1] = s[6];
        p[2] = '\0';

        i = strtol(p, NULL, 16);

        return (float) i / 255;
}

#ifdef _UNIT_TEST_
int main(void)
{
        char *ptr = "FF";
        int val;

        printf("White=%s\n", picviz_color_named_to_hexstr("white"));
        printf("Red=%s\n", picviz_color_named_to_hexstr("red"));
        printf("Red value of #FF0000 = %f\n", picviz_color_extract_r("#FF0000"));
        printf("Red value of #eA0000 = %f\n", picviz_color_extract_r("#eA0000"));
        printf("Green value of #eABF00 = %f\n", picviz_color_extract_g("#eABF00"));

}
#endif
